//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormPrincipal *FormPrincipal;
//---------------------------------------------------------------------------
__fastcall TFormPrincipal::TFormPrincipal(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TFormPrincipal::FormCreate(TObject *Sender)
{
  Umbral = 128;

  reader = ImageReaderType::New();  // create the smart pointer
  reader->SetFileName("test.png");
  reader->Update();

  // Instantiate a filter and attach it to the reader.
  binario = ThresholdFilterType::New();
  binario->SetLowerThreshold(0);
  binario->SetUpperThreshold(Umbral);
  binario->SetInsideValue(255);
  binario->SetOutsideValue(0);

  // Instantiate a Danielsson Distance Map Filter.
  filter = FilterType::New();
  filter->InputIsBinaryOn();

  transform = TransformType::New();
  interpolator = InterpolatorType::New();

  double spacing[ Dimension ];
  spacing[0] = 1; // pixel spacing in millimeters along X
  spacing[1] = 1; // pixel spacing in millimeters along Y

  double origin[ Dimension ];
  origin[0] = 1;  // X space coordinate of origin
  origin[1] = 1;  // Y space coordinate of origin

  ImageType::RegionType Region = reader->GetOutput()->GetLargestPossibleRegion();
  size = Region.GetSize();

  resample = ResampleType::New();
  resample->SetInterpolator( interpolator.GetPointer() );
  resample->SetTransform( transform.GetPointer() );
  resample->SetDefaultPixelValue( 0 );
  resample->SetOutputSpacing( spacing );
  resample->SetOutputOrigin( origin );
  resample->SetSize( size );


  // Instantiate a Rescaler Filter.
  scaler = RescalerType::New();
  scaler->SetOutputMinimum(  50L  );
  scaler->SetOutputMaximum( 240L );

  // Instantiate a Rescaler Filter.
  caster = CastType::New();


  // Instantiate a writer and attach it to the filter.
  writer = ImageWriterType::New();
  writer->SetImageIO(itk::PNGImageIO::New());
  writer->SetFileName("output256.png");

    // Instantiate a writer for the scaler
  scalerwriter = WriterType::New();
  scalerwriter->SetImageIO(itk::PNGImageIO::New());
  scalerwriter->SetFileName( "daniel.png" );


  binario->SetInput(reader->GetOutput());
  resample->SetInput( binario->GetOutput() );
  filter->SetInput( resample->GetOutput() );
  scaler->SetInput( filter->GetOutput() );
  caster->SetInput( scaler->GetOutput() );
  writer->SetInput(binario->GetOutput());
  scalerwriter->SetInput(scaler->GetOutput());

  scaler->Update();
  scalerwriter->Update();
  writer->Update();
}
//---------------------------------------------------------------------------
__fastcall TFormPrincipal::~TFormPrincipal()
{
}
//---------------------------------------------------------------------------
void __fastcall TFormPrincipal::SalirClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TFormPrincipal::DanielssonClick(TObject *Sender)
{
  ImageType::Pointer ItkImage = reader->GetOutput();

  IteratorType it(ItkImage, ItkImage->GetLargestPossibleRegion());

  // Viewport
  int TamanioBuffer = ItkImage->GetPixelContainer()->Size();
  Graphics::TBitmap *bmp = new Graphics::TBitmap;
  bmp->PixelFormat = pf32bit;
  bmp->Width = size[0];
  bmp->Height = size[1];

  it.GoToBegin();
  unsigned int *bPixels = (unsigned int*)bmp->ScanLine[0];

  int Columna = 0;
  int Fila = size[1];
  while(!it.IsAtEnd())   {
    if (Columna < size[0]) {
      bPixels[Columna] = RGB(it.Get(), it.Get(), it.Get());
      Columna++;
      ++it;
    } else {
      Columna = 0;
      Fila--;
      bPixels = (unsigned int*)bmp->ScanLine[Fila];
      bPixels[Columna] = RGB(it.Get(), it.Get(), it.Get());
    }
  }
  ImagenVisual->Canvas->Draw(0, 0, bmp);
  delete bmp;
}
//---------------------------------------------------------------------------
void __fastcall TFormPrincipal::FormClose(TObject *Sender, TCloseAction &Action)
{
  reader = 0;
  binario = 0;
  filter = 0;
  scaler = 0;
  caster = 0;
  writer = 0;
  scalerwriter = 0;
  interpolator = 0;
  transform = 0;
  resample = 0;

}
//---------------------------------------------------------------------------

